@@ -46,6 +46,8 @@
 #include <libindicate/indicator.h>
 #include <libindicate/indicator-messages.h>
 
+#include <unity.h>
+
 #include "xutils.h"
 
 #define CONF_DIR          "/apps/evolution/eplugin/evolution_indicator"
@@ -71,7 +73,8 @@ static IndicateServer *server     = NULL;
 static GSList         *indicators = NULL;
 static gint            n_accounts = 0;
 
-static IndicateIndicator  *pop_indicator  = NULL;
+static UnityLauncherEntry       *launcher = NULL;
+
 static NotifyNotification *notification   = NULL;
 static ca_context         *canberra_cxt   = NULL;
 static ca_proplist        *canberra_props = NULL;
@@ -152,7 +155,11 @@ evolution_is_focused (void)
   }
 
   xwindow = None;
-  res = _wnck_get_window (GDK_WINDOW_XWINDOW (root), 
+#if GTK_CHECK_VERSION(2, 91, 0)
+  res = _wnck_get_window (GDK_WINDOW_XID (root), 
+#else
+  res = _wnck_get_window (GDK_WINDOW_XWINDOW (root),
+#endif
                     gdk_x11_get_xatom_by_name ("_NET_ACTIVE_WINDOW"),
                     &xwindow);
   if (res)
@@ -220,14 +227,18 @@ set_indicator_unread_count (IndicateIndicator *indicator, gint count)
 void
 org_gnome_mail_new_notify (EPlugin *ep, EMEventTargetFolder *t)
 {
-	g_return_if_fail (t != NULL);
+  gchar *url;
+
+  g_return_if_fail (t != NULL);
+
+  url = camel_service_get_url (CAMEL_SERVICE (t->store));
 
   if (!t->new)
 		return;
 
   if (only_inbox && !(t->is_inbox))
   {
-    g_debug ("EI: %s is not an inbox", t->uri);
+    g_debug ("EI: %s is not an inbox", url);
     return;
   }
 
@@ -239,7 +250,7 @@ org_gnome_mail_new_notify (EPlugin *ep, EMEventTargetFolder *t)
 
 	g_static_mutex_lock (&mlock);
 
-  g_debug ("EI:mail_new_notify: %s", t->uri);
+  g_debug ("EI:mail_new_notify: %s", url);
 
   message_count += t->new;
  
@@ -252,7 +263,7 @@ org_gnome_mail_new_notify (EPlugin *ep, EMEventTargetFolder *t)
       {
         IndicateIndicator *indi = i->data;
 
-        if (g_strstr_len (t->uri,
+        if (g_strstr_len (url,
                           -1,
                           indicate_indicator_get_property (indi, "url")))
           {
@@ -260,9 +271,6 @@ org_gnome_mail_new_notify (EPlugin *ep, EMEventTargetFolder *t)
             break;
           }
       }
-    if (!indicator && pop_indicator) /* Use pop (default) indicator */
-      indicator = pop_indicator;
-    
     if (indicator)
       {
         gint count;
@@ -276,10 +284,12 @@ org_gnome_mail_new_notify (EPlugin *ep, EMEventTargetFolder *t)
       }
     else
       {
-        g_warning ("EI: Unable to find account that matches %s", t->uri);
+        g_warning ("EI: Unable to find account that matches %s", url);
       }
     }
 
+  update_unity_launcher_count ();
+
   /* Show bubble */
   if (show_bubble)
   {
@@ -289,7 +299,7 @@ org_gnome_mail_new_notify (EPlugin *ep, EMEventTargetFolder *t)
 
     if (notification == NULL)
     {
-      notification = notify_notification_new (" ", " ", "mail-unread", NULL);
+      notification = notify_notification_new (" ", " ", "mail-unread");
     }
 
     trans = g_dngettext (PACKAGE, 
@@ -362,9 +372,37 @@ org_gnome_mail_read_notify (EPlugin *ep, EMEventTargetMessage *t)
     }
   message_count = 0;
 
+  update_unity_launcher_count ();
+
   g_static_mutex_unlock (&mlock);
 }
 
+void
+update_unity_launcher_count ()
+{
+  GSList *i;
+  int count = 0;
+
+  g_debug ("EI: update_unity_launcher_count");
+
+  for (i = indicators; i; i = i->next)
+  {
+    IndicateIndicator *indicator = i->data;
+
+    count = count + get_indicator_unread_count (indicator);
+
+    g_debug ("EI: Setting count to %d unread messages", count);
+
+  }
+  unity_launcher_entry_set_count (launcher, count);
+  if (count > 0)
+  {
+    unity_launcher_entry_set_count_visible (launcher, TRUE);
+  } else {
+    unity_launcher_entry_set_count_visible (launcher, FALSE);
+  }
+}
+
 /*
  * GCONF CLIENTS
  */
@@ -603,21 +641,7 @@ update_accounts (void)
                                         NULL);
 
           /* Check to see account already exists and, if not, create it */
-          if (data.url && g_str_has_prefix (data.url, "pop:"))
-            {
-              if (pop_indicator == NULL)
-                {
-                  pop_indicator = create_indicator ("pop:",
-                                                  g_dgettext ("evolution-2.28",
-                                                                "Inbox"));
-                  g_debug ("EI: New Inbox indicator");
-                }
-              indicator = pop_indicator;
-            }
-          else
-            {
-              indicator = find_indicator_for_url (indicators, data.url);
-            }
+          indicator = find_indicator_for_url (indicators, data.url);
           if (indicator)
             {
               old_list = g_slist_remove (old_list, indicator);
@@ -625,7 +649,16 @@ update_accounts (void)
             }
           else
             {
-              indicator = create_indicator (data.url, data.name);
+              if (data.url && g_str_has_prefix (data.url, "pop:"))
+                {
+                  indicator = create_indicator ("pop:",
+                                                g_dgettext (EVO_I18N_DOMAIN,
+                                                            "Inbox"));
+                }
+              else
+                {
+                  indicator = create_indicator (data.url, data.name);
+                }
               indicators = g_slist_append (indicators, indicator);
 
               g_debug ("EI: New account: %s (%s)", data.name, data.url);
@@ -740,6 +773,8 @@ e_plugin_lib_enable (EPlugin *ep, int enable)
 	dbusmenu_server_set_root(menu_server, root);
 	indicate_server_set_menu(server, menu_server);
 
+    launcher = unity_launcher_entry_get_for_desktop_file (EVOLUTION_DESKTOP_FILE);
+
     client = gconf_client_get_default ();
     gconf_client_add_dir (client, CONF_DIR, GCONF_CLIENT_PRELOAD_NONE, NULL);
 
@@ -890,10 +925,17 @@ org_gnome_get_prefs (EPlugin *ep, EConfigHookItemFactoryData *data)
 
     label2 = gtk_label_new (" ");
 
+#if GTK_CHECK_VERSION(2, 91, 0)
+    combo = gtk_combo_box_text_new ();
+    gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combo),
+				    n_accounts > 1 ? _("any Inbox") : _("Inbox"));
+    gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combo), _("any Folder"));
+#else
     combo = gtk_combo_box_new_text ();
     gtk_combo_box_append_text (GTK_COMBO_BOX (combo),
                                n_accounts > 1 ? _("any Inbox") : _("Inbox"));
     gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _("any Folder"));
+#endif
     gtk_combo_box_set_active (GTK_COMBO_BOX (combo), 
                               only_inbox ? 0 : 1);
     g_signal_connect (combo, "changed", G_CALLBACK (on_combo_changed), NULL);
@@ -1007,6 +1049,7 @@ show_evolution (gpointer arg0, guint timestamp, gpointer arg1)
 
     }
     message_count = 0;
+    update_unity_launcher_count ();
   }
   else
   {
